#!/bin/bash
#
# creates backups of essential files - (c) Paolo Subiaco http://www.creasol.it
#
# Sintax: backup.sh [CMD]
# where CMD can be:
# -weekly				weekly complete backup
# -tosmb				just export data to smb share, if BACKUP_TO_SMB -eq 1
# -tousb				just export data to usb disk, if BACKUP_TO_USB -eq 1
# -toftp				just export data to ftp server, if BACKUP_TO_FTP -eq 1

if [ ! -f /usr/local/etc/backup.conf ]; then
	echo "Configuration file /usr/local/etc/backup.conf missing"
	echo "Configuration file /usr/local/etc/backup.conf missing" |mail -s "backup error" psubiaco@creasol.it
	exit
fi

. /usr/local/etc/backup.conf

export PATH='/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin:/usr/local/sbin'
export LC_TIME="it_IT" 2>/dev/null
export LC_TIME="it_IT@euro" 2>/dev/null
export LANG="it_IT.UTF-8"

#Variables used to scan backup.smb file
nl='
'
tab='	'
space=' '

if [ -z "${COMPRESS}" ]; then
	COMPRESS="gzip"
	COMPRESS_ARG="-c"
	COMPRESS_SUFFIX="tar.gz"
fi

CCRYPT=""
if [ -x /usr/local/sbin/ccrypte ]; then
	COMPRESS="/usr/local/sbin/ccrypte"
	COMPRESS_ARG=""
	COMPRESS_SUFFIX="tar.gzc"
fi

function errorsinc () {
	errors=$(( $errors + 1 ))
}

function printlog () {
	logtype=$1
	shift
	echoargs=''
	if [ "$1" == "-n" ]; then
		echoargs='-n'
		shift
	fi
	if [ "$logtype" = "info" ]; then
		echo -e ${echoargs} "$*" >>${INFOFILE}
	else
		echo -e ${echoargs} "$*" >>${LOGFILE}
		echo -e ${echoargs} "WARN: $*" >>${INFOFILE}
	fi
}

function printerror () {
	# update LOGFILE, send email and exit
	# $1: email subject
	# $2: another information about the current error
	printlog warn $1
	echo -e "$2\n\n`cat ${LOGFILE}`" | mail -s "BACKUP System: $1" ${EMAIL_ADMIN} 
	exit 1
}

function copyFileIfNew () {
	# syntax: copyFileIfNew FILE_SOURCE  PATH_DESTINATION/FILENAME 
	# Copy the file only if the destination path does not contain it
	# or if it is different in size
	if [ $# -ne 2 ]; then
		printlog warn "Error invoking copyFileIfNew $*"
		return
	fi
	if [ -f "$2" ]; then
		filesizes="`ls -l $1 |awk '{print $5}'`"
		filesized="`ls -l $2 |awk '{print $5}'`"
		if [ $filesizes -eq $filesized ]; then
			# file size is the same => don't copy 
			return	
#		else
#			echo "file $1: size differs: src=$filesizes, dst=$filesized"
		fi
	fi
	#nice -$NICE cp -v --preserve=mode,ownership $1 $2 >>$INFOFILE 2>>$LOGFILE
	nice -$NICE cp -v --preserve=timestamps $1 $2 >>$INFOFILE 2>>$LOGFILE
}

function purgedir () {
	echo "Remove from $1/logs files older than $LOGS_MAXBACKUPTIME days" >>${INFOFILE}
	nice -$NICE find $1/logs -type f -mtime +$LOGS_MAXBACKUPTIME -exec rm {} \; 2>>${LOGFILE} >/dev/null
	echo "Remove from $1/daily-weekly-mysql files older than $2 days" >>${INFOFILE}
	for d in daily weekly mysql; do
		nice -$NICE find $1/$d -type f -mtime +$2 -exec rm {} \; 2>>${LOGFILE} >/dev/null
	done
}

function backup_logs () {
	if [ ! -d "${BACKUP_DIR}/logs" ]; then
		mkdir -p "${BACKUP_DIR}/logs"  2>>${LOGFILE}
		chgrp backup ${BACKUP_DIR}/logs
		chmod 2750 ${BACKUP_DIR}/logs
	fi
	if [ "$BACKUP_LOGS" != 1 ]; then return; fi
	# perform backup of logs...
	# Remove old backups
	nice -$NICE find ${BACKUP_DIR}/logs -type f -mtime +$LOGS_MAXBACKUPTIME -exec rm {} \; 2>>${LOGFILE} >/dev/null
	> $LIST
	cd /
	for file in audit/audit.log apache2/access.log apache2/error.log	asterisk/full	asterisk/messages	auth.log	daemon.log	dansguardian/access.log	lastlog	messages	proftpd/proftpd.log	proftpd/xferlog	squid/access.log	syslog	user.log	wtmp dpkg.log ; do
		if [ -f var/log/$file.0 ]; then
			echo var/log/$file.0 >>$LIST
		elif [ -f var/log/$file.0.gz ]; then
			echo var/log/$file.0.gz >>$LIST
		elif [ -f var/log/$file.1 ]; then
			echo var/log/$file.1 >>$LIST
		elif [ -f var/log/$file.1.gz ]; then
			echo var/log/$file.1.gz >>$LIST
		fi
	done
	if [ -s $LIST ]; then
		nice -$NICE tar cpT "$LIST" 2>>"$LOGFILE" |${COMPRESS} ${COMPRESS_ARG} >"${BACKUP_DIR}/logs/backup_logs_$filesuffix.${COMPRESS_SUFFIX}" 
	fi
}

function tape_status () {
  TAPESTAT="Unknown"
	TAPE_STATUS=`mt -f $TAPE_DEVICE status 2>&1 | tr "\n" " "`
	if [ -n "`echo "$TAPE_STATUS" | grep '.*Input.output error.*'`" ]; then
		TAPESTAT="IOerror"
	elif [ -n "`echo "$TAPE_STATUS" | grep '.*Device or resource busy.*'`" ]; then
		TAPESTAT="Busy"
	elif [ -n "`echo "$TAPE_STATUS" | grep '.*No medium found.*'`" ]; then
		TAPESTAT="NoTape"
	elif [ -n "`echo "$TAPE_STATUS" | grep '.*BOT ONLINE .*'`" ]; then
		TAPESTAT="Begin"
	fi

	export TAPESTAT
	export TAPE_STATUS
}


function tape_backup () {
  # weekly backup on DAT
	printlog info "============= Start backup to TAPE ==============="
	tape_step="Tape.Iniziale:"
	retry=20
	while [ $retry -gt 0 ]; do
		let retry--
		tape_status
		case $TAPESTAT in
			NoTape)
				errorsinc #Increment variable errors
				printerror "$tape_step Tape non inserita";;
			IOerror)
				sleep 5;;
			Begin)
				break;;
			*)
				errorsinc #Increment variable errors
				printerror "$tape_step Unknown status";;
		esac
	done
	if [ $retry == 0 ]; then
		errorsinc #Increment variable errors
		printerror "$tape_step Superato numero max tentativi"
	fi

	tape_step="Tape.Scrivi"
	retry=20
	tar c `echo "$BACKUP_DIR" 2>>${LOGFILE} |cut -b 2-`/weekly/* |dd of=$TAPE_DEVICE
	while [ $retry > 0 ]; do
		let retry--
		sleep 10
		tape_status
		if [ "$TAPESTAT"=="Begin" ]; then
			break;
		fi
	done
	if [ $retry == 0 ]; then
		errorsinc #Increment variable errors
		printerror "$tape_step Superato numero max tentativi"
	fi

	#begin comparation
	echo "Lista dei file su tape (controllare eventuali corruzioni):"
	tar tf $TAPE_DEVICE 2>>${LOGFILE} 
	mt -f $TAPE_DEVICE rewoffl
}

function cdrw_backup () {
	# start writing image and backup on cdrom or dvd
	echo "============= Start backup to CD/DVD ===============" >>$LOGFILE
	umount  ${CDRW_DEVICE} >/dev/null 2>&1
	eject -t ${CDRW_DEVICE} >/dev/null 2>&1
	if [ "$BACKUP_TO_DVDRW" = "1" ]; then
		# format dvd
		#$DVDFORMAT -gui -blank ${CDRW_DEVICE}
		if [ "$DVD_MIN_SIZE" != "0" ]; then
			size=`du "${BACKUP_DIR}/weekly" |cut -f 1`
			if [ $size -lt $DVD_MIN_SIZE ]; then
				dd if=/dev/zero of="${BACKUP_DIR}/weekly/dvd_pad" count=$DVD_MIN_SIZE ibs=1024
			fi
		fi				
		# create ISO then burn
		$GROWISOFS -gui -Z ${CDRW_DEVICE} -R -J -full-iso9660-filenames -iso-level 2 -sysid LINUX "${BACKUP_DIR}/weekly" 2>>"$LOGFILE"
	else 
		#	erase CDRW
		$CDRECORD dev=${CDRW_SCSI_DEVICE} speed=${CDRW_SPEED} blank=fast --force 2>>"$LOGFILE" 1>&2
		# Write RockRidge image
		if [ -z "$CDRW_DIRS_FILE" ]; then
			# Write all files in weekly/
			nice -$NICE $MKISOFS -v "Backup_$6_$2_$3" -pad -P "Paolo Subiaco" -p "info@creasol.it" \
					-o "${BACKUP_CDRW_IMAGE}" -l -r "${BACKUP_DIR}/weekly" 2>>"$LOGFILE"
		else
			# Write only files specified in $CDRW_DIRS_FILE
			dirs=""
			for file in `cat $CDRW_DIRS_FILE`; do
				dirs="$dirs ${BACKUP_DIR}/weekly/$file"
			done
			echo "backup only files `ls $dirs`" >>"$LOGFILE"
			$MKISOFS -V "Backup_$6_$2_$3" -pad -P "Paolo Subiaco" -p "info@creasol.it" \ 
					-o "${BACKUP_CDRW_IMAGE}" -l -r `ls $dirs`  2>>"$LOGFILE"
		fi	
					
		# write CDRW
		$CDRECORD dev=${CDRW_SCSI_DEVICE} speed=${CDRW_SPEED} "${BACKUP_CDRW_IMAGE}" \
				2>>"$LOGFILE" 1>&2
		sleep 10
		eject $CDRW_DEVICE
		sleep 5
		eject -t $CDRW_DEVICE
		sleep 10

		# verify CDRW
		if [ -f /tmp/cdrw -o -d /tmp/cdrw ]; then
			rm -rf /tmp/cdrw 2>/dev/null
		fi
		mkdir /tmp/cdrw 2>/dev/null
		mount -o ro "${CDRW_DEVICE}" /tmp/cdrw 2>/dev/null
		cd "${BACKUP_DIR}/weekly"
		if [ -z "$CDRW_DIRS_FILE" ]; then
			files=`ls backup*`
		else
			files=`cat $CDRW_DIRS_FILE`
			files=`ls $files`
		fi
		filediffer=""
		for file in $files; do 
			if [ `diff $file /tmp/cdrw/$file` ]; then
				sleep 30
				if [ `diff $file /tmp/cdrw/$file` ]; then
					filediffer="${filediffer}Archivio non scritto correttamente: ${file}\n"
				fi
			fi
		done
		# Remove RockRidge image
		rm -f "${BACKUP_CDRW_IMAGE}"
		cd /
		umount /tmp/cdrw
		if [ -n "$filediffer" ]; then
			printerror "Errore masterizzazione" "${filediffer}"
		fi
	fi	
	eject ${CDRW_DEVICE}
}

function backup_from_smb () {
	# backup SMB shares 
	backup_from_smb_type="$1"
	if [ "$backup_from_smb_type" != "incremental" ]; then
		backup_from_smb_type="weekly"
	fi
	
	if [ ! -f /usr/local/etc/backup.smb ]; then
		return
	fi
	smb_dir=/mnt/fromsmb
	if [ ! -d $smb_dir ]; then
		mkdir -p ${smb_dir}
	fi
	IFS="$nl"
	for line in `cat /usr/local/etc/backup.smb|egrep -v '^#'`; do
		if [ -n "`grep IFS=: /usr/local/etc/backup.smb`" ]; then
			IFS=:
		else
			IFS="$space$tab$nl"
		fi
		set $line
		IFS="$space$tab$nl"
		if [ $# -eq 6 ]; then
			# $1=workgroup, $2=smbshare, $3=username, $4=password, $5=outputDir, $6=outputFile
			dir="${BACKUP_DIR}/$5"
			if [ "$backup_from_smb_type" == "weekly" ]; then
				dir="$dir/weekly"
			else
				dir="$dir/daily"
			fi
			# Control that destination directory exists
			if [ ! -d $dir ]; then mkdir -p $dir; fi
			dirfile="$dir/$6_${backup_from_smb_type}_$filesuffix.${COMPRESS_SUFFIX}"
			printlog info "Backup SMB share $2" 
			mount -t cifs "$2" ${smb_dir} -o username=$3,password=$4,domain=$1,ro 2>&1
			if [ -n "`mount |grep ${smb_dir} |grep -v grep`" ]; then	
				# No errors: share correctly mounted!
				oldpwd=`pwd`
				cd ${smb_dir}
				if [ -z "{BACKUP_FROM_SMB_MAXBACKUPTIME}" ]; then
					BACKUP_FROM_SMB_MAXBACKUPTIME=30
				fi
				purgedir $dir/.. ${BACKUP_FROM_SMB_MAXBACKUPTIME}
				if [ "$backup_from_smb_type" == "weekly" ]; then
					# weekly backup 
					rm -f "$dir/$6_${backup_from_smb_type}_*.${COMPRESS_SUFFIX}" 2>>"$LOGFILE"
					nice -$NICE tar cp $EXCLUDE --exclude nobackup . 2>>${LOGFILE} |${COMPRESS}  ${COMPRESS_ARG} >"${dirfile}" 2>>$LOGFILE
					ls -l ${dirfile}
				else
					# incremental backup
					find . -depth -type f -mmin -1800 -print > $LIST 2>>${LOGFILE}
					if [ -s "${LIST}" ]; then 
						nice -$NICE tar cpT "$LIST" $EXCLUDE  --exclude nobackup  2>>${LOGFILE} |${COMPRESS} ${COMPRESS_ARG} >"${dirfile}" 2>>$LOGFILE
						ls -l ${dirfile}
					fi
					rm -f "$LIST"
				fi
				cd $oldpwd
				sleep 15
				umount $smb_dir	2>>${LOGFILE}
				sleep 15
				umount $smb_dir 2>/dev/null
				sleep 15
			else
				if [ "a$BACKUP_FROM_SMB_ALERT" != "a0" -o "$cmd" == "-fromsmb" ]; then
					errorsinc #Increment variable errors
					printlog warn "Error mounting SMB share: command mount -t cifs $2 $smb_dir -o username=$3,password=$4,domain=$1,ro\n$RESULT"
				else
					printlog info "Impossibile montare la condivisione $2"
				fi
			fi
		fi
		IFS="$nl"
	done
	IFS="$space$tab$nl"
}

function smb_backup () {
	if [ "$BACKUP_TO_SMB" = 1 ]; then
		printlog info "============= Start backup to SMB share at `date` ==============="
		if [ -n "$SMB_PASS" ]; then
			SMBPASS="password=$SMB_PASS"
		else
			SMBPASS="guest"
		fi
		if [ -n "$SMB_USER" ]; then
			SMBUSER="-o username=$SMB_USER,$SMBPASS,sec=ntlm"
		else
			SMBUSER="-o guest"
		fi
		if [ -n "$SMB_DOMAIN" ]; then
			SMBUSER="$SMBUSER,domain=${SMB_DOMAIN}"
		fi
		if [ -n "`mount | grep ${SMB_DIR}`" ]; then
			printlog info "First, umount already mounted $SMB_DIR"
			$SMBUMOUNT $SMB_DIR 2>>$LOGFILE >>${INFOFILE}
			sleep 5
		fi
		RESULT=`$SMBMOUNT $SMB_SHARE $SMB_DIR $SMBUSER 2>/dev/null`
		if [ -z "`mount |grep $SMB_DIR |grep -v grep`" ]; then
			#retry mounting smb partition
			sleep 2
			RESULT=`$SMBMOUNT $SMB_SHARE $SMB_DIR $SMBUSER 2>/dev/null`
			if [ -z "`mount |grep $SMB_DIR |grep -v grep`" ]; then
				#retry mounting smb partition
				sleep 2
				RESULT=`$SMBMOUNT $SMB_SHARE $SMB_DIR $SMBUSER 2>/dev/null`
			fi
		fi
		if [ -n "`mount |grep $SMB_DIR |grep -v grep`" ]; then	
			# No errors: share correctly mounted!
			for dir in mysql weekly daily logs ${BACKUP_OTHER_DIRS}; do
				if [ -f "$SMB_DIR/$SMB_SUBDIR/$dir" ]; then
					rm "$SMB_DIR/$SMB_SUBDIR/$dir"
				fi
				if [ ! -d $SMB_DIR/$SMB_SUBDIR/$dir ]; then 
					printlog info "Create SMB subdirectory $dir ..."
					mkdir -p "$SMB_DIR/$SMB_SUBDIR/$dir" 2>>${LOGFILE}
				fi
			done
			smb_backup_pre 2>/dev/null
			#remove old backups
			#nice -$NICE find $SMB_DIR/$SMB_SUBDIR -type f -mmin +$(( $SMB_MAXBACKUPTIME * 24 * 60 - 720 )) -exec rm {} \; 2>>${LOGFILE} >/dev/null
			purgedir $SMB_DIR/$SMB_SUBDIR $SMB_MAXBACKUPTIME
			#nice -$NICE cp -uR $BACKUP_DIR/* $SMB_DIR/$SMB_SUBDIR
			for dir in mysql weekly daily logs ${BACKUP_OTHER_DIRS}; do
				cd $BACKUP_DIR/$dir
	#			for file in `find . -type f -mtime -$SMB_MAXBACKUPTIME |egrep '^\./(weekly|daily|logs)'`; do	;#only local backup
				for file in `find . -type f -mtime -$SMB_MAXBACKUPTIME`; do	#both local and other backups
					filecp="`echo $file |sed 's/^\.\///'`"
					filesize="`ls -l $filecp |awk '{print $5}'`"
					if [ "$SMBMOUNT"!="smbmount" -o $filesize -lt 2147483647 ]; then	
						copyFileIfNew $filecp $SMB_DIR/$SMB_SUBDIR/$dir/$filecp
					else
						#file bigger than 2GB => split?
						if [ -f $SMB_DIR/$SMB_SUBDIR/$dir/${filecp}_aa ]; then
							printlog info "Skip ${filecp} because ${filecp}_aa already exists!" 
						else
							split -b 2000000000 ${filecp} ${filecp}_
							for file in ${filecp}_??; do 
								copyFileIfNew $file $SMB_DIR/$SMB_SUBDIR/$dir/$file
								rm $file
							done
						fi
					fi
				done
			done
			smb_backup_post 2>/dev/null
			sleep 16
			$SMBUMOUNT $SMB_DIR 2>>${LOGFILE}
		else
			errorsinc #Increment variable errors
			printlog warn "Error mounting SMB share: command $SMBMOUNT $SMB_SHARE $SMB_DIR $SMBUSER"
			$SMBMOUNT $SMBSHARE $SMB_DIR $SMBUSER 2>&1 >>${INFOFILE}
			if [ -n "$SMB_HOST" ]; then
				ping -c 1 $SMB_HOST
				ping -c 1 $SMB_HOST
				if [ $? -ne 0 ]; then
					# host does not reply to ping
					printlog warn "SMB server $SMB_HOST does not reply to ping"
					if [ -n "$SMB_MAIL" ]; then
						# send email to advise about SMB host
						echo "$SMB_BODY" | mail -s "$SMB_SUBJECT" $SMB_MAIL
					fi
				fi
			else
				printlog warn "SMB server $SMB_HOST does not reply on SMB service"
				if [ -n "$SMB_MAIL" ]; then
					# send email to advise about SMB host
					echo "$SMB_BODY" | mail -s "$SMB_SUBJECT" $SMB_MAIL
				fi
			fi
		fi
	fi
}

function smb_backup2 () {
	# Make backup to another SMB server
	BACKUP_TO_SMB=${BACKUP_TO_SMB2}
	SMB_MAXBACKUPTIME=${SMB_MAXBACKUPTIME2}
	SMB_SHARE=${SMB_SHARE2}
	SMB_DIR=${SMB_DIR2}
	SMB_USER=${SMB_USER2}
	SMB_PASS=${SMB_PASS2}
	SMB_DOMAIN=${SMB_DOMAIN2}
	SMB_SUBDIR=${SMB_SUBDIR2}
	SMBMOUNT=${SMBMOUNT2}
	SMBUMOUNT=${SMBUMOUNT}
	SMB_HOST=${SMB_HOST2}
	SMB_MAIL=${SMB_MAIL2}
	SMB_SUBJECT=${SMB_SUBJECT2}
	SMB_BODY=${SMB_BODY2}
	smb_backup
}	


function partition_backup () {
	if [ "$BACKUP_TO_PARTITION" = 1 ]; then
		printlog info "================= Start backup to partition at `date` =============="
		if [ ! -d $PARTITION_DIR ]; then
			mkdir -p $PARTITION_DIR  2>>${LOGFILE}
			chown backup $PARTITION_DIR 2>/dev/null
			chmod 700 $PARTITION_DIR
		fi
		$PARTITION_MOUNT
		if [ -n "`mount |grep $PARTITION_DIR`" ]; then
			# partition mounted
			if [ ! -d $PARTITION_DIR/$PARTITION_SUBDIR ]; then
				mkdir -p $PARTITION_DIR/$PARTITION_SUBDIR  2>>${LOGFILE}
			else
				# Remove old backups
				#nice -$NICE find $PARTITION_DIR -type f -mmin +$(( $PARTITION_MAXBACKUPTIME * 24 * 60 - 720 )) -exec rm {} \; 2>>${LOGFILE} >/dev/null
				purgedir $PARTITION_DIR/$PARTITION_SUBDIR $PARTITION_MAXBACKUPTIME 
			fi
			#nice -$NICE cp -uRpv $BACKUP_DIR/* $PARTITION_DIR/$PARTITION_SUBDIR >>${INFOFILE} 2>>${LOGFILE}
			for dir in weekly daily logs mysql ${BACKUP_OTHER_DIRS}; do
				if [ -f $PARTITION_DIR/$PARTITION_SUBDIR/$dir ]; then
					rm $PARTITION_DIR/$PARTITION_SUBDIR/$dir 
				fi
				if [ ! -d $PARTITION_DIR/$PARTITION_SUBDIR/$dir ]; then
					mkdir -p $PARTITION_DIR/$PARTITION_SUBDIR/$dir  2>>${LOGFILE}
				fi
				cd $BACKUP_DIR/$dir
				for file in `find -maxdepth 2 -type f -mtime -$PARTITION_MAXBACKUPTIME`; do
					file="`echo $file |sed 's/^\.\///'`"
					nice -$NICE cp -uv --preserve=timestamps $file $PARTITION_DIR/$PARTITION_SUBDIR/$dir/ >>${INFOFILE} 2>${LOGFILE}
				done
			done
			echo "Partition usage:" >>${INFOFILE}
			df |head -n 1 >>${INFOFILE}
			df |grep ${PARTITION_DIR} >>${INFOFILE}
			echo "=========================================================" >>${INFOFILE}
		else
			# Error mounting partition
			errorsinc #Increment variable errors
			if [ "${cmd}"=="-topartition" ]; then
				#Error mounting partition: skip this error
				return
			fi
			printerror "Error mounting partition $PARTITION_DEVICE"
		fi
		$PARTITION_UMOUNT	#umount partition, if needed
	fi
}


function usb_backup () {
	if [ "$BACKUP_TO_USB" = 1 ]; then
		IFS="$space$tab$nl"
		printlog info "================= Start backup to USB disk at `date` =============="
		if [ ! -d $USB_DIR ]; then
			mkdir -p $USB_DIR  2>>${LOGFILE}
			chown backup $USB_DIR 2>/dev/null
			chmod 700 $USB_DIR
		fi
		if [ -n "$USB_MODULES" ]; then
			for mod in $USB_MODULES; do 
				modprobe -r $mod
			done
			for mod in $USB_MODULES; do 
				modprobe $mod
			done
		fi
		if [ -n "${USB_PREEXEC}" ]; then
			bash -c "${USB_PREEXEC}"
		fi
		sleep 10
		if [ -z "$USB_UUID" ]; then
			USB_UUID=$USB_DEVICE
		fi
		mount -o async,noexec,nodev $USB_UUID $USB_DIR 2>&1 >>${INFOFILE}
		sleep 5
		if [ -n "`mount |grep $USB_DIR`" ]; then
			# USB disk mounted
			if [ ! -d $USB_DIR/$USB_SUBDIR ]; then
				mkdir -p $USB_DIR/$USB_SUBDIR  2>>${LOGFILE}
			else
				# Remove old backups
				#nice -$NICE find $USB_DIR -type f -mmin +$(( $USB_MAXBACKUPTIME * 24 * 60 - 720 )) -exec rm {} \; 2>>${LOGFILE} >/dev/null
				purgedir ${USB_DIR}/${USB_SUBDIR} $USB_MAXBACKUPTIME
			fi
			usb_backup_pre 2>/dev/null
			#nice -$NICE cp -uRpv $BACKUP_DIR/* $USB_DIR/$USB_SUBDIR >>${INFOFILE} 2>>${LOGFILE}
			for dir in weekly daily logs mysql ${BACKUP_OTHER_DIRS}; do
				if [ -f $USB_DIR/$USB_SUBDIR/$dir ]; then
					rm $USB_DIR/$USB_SUBDIR/$dir 
				fi
				if [ ! -d $USB_DIR/$USB_SUBDIR/$dir ]; then
					mkdir -p $USB_DIR/$USB_SUBDIR/$dir  2>>${LOGFILE}
				fi
				cd $BACKUP_DIR/$dir
				for file in `find -maxdepth 2 -type f -mtime -$USB_MAXBACKUPTIME`; do
					file="`echo $file |sed 's/^\.\///'`"
					nice -$NICE cp -uv --preserve=mode,timestamps $file $USB_DIR/$USB_SUBDIR/$dir/ >>${INFOFILE} 2>${LOGFILE}
				done
			done
			usb_backup_post 2>/dev/null
			echo "Disk usage on USB device:" >>${INFOFILE}
			df |head -n 1 >>${INFOFILE}
			df |grep ${USB_DIR} >>${INFOFILE}
			usage=`df |grep ${USB_DIR} |awk '{print $5}'|tr -d %`
			echo "usate=$usage"
			if [ -n "$usage" ]; then
				if [ $usage -gt 90 ]; then
					errorsinc #Increment variable errors
					printlog warn "USB partition ${USB_DIR} is full"
					df |mail -s "backup: USB partition ${USB_DIR} full" $EMAIL_ADMIN
				fi
			fi
			echo "=========================================================" >>${INFOFILE}
		else
			# Error mounting USB device
			errorsinc #Increment variable errors
			if [ "${cmd}"=="-tousb" ]; then
				#Error mounting USB device: skip this error
				return
			fi
			printerror "Error mounting USB device $USB_UUID"
			printerror "Command was: mount -o async,noexec,nodev $USB_UUID $USB_DIR "
		fi
		umount $USB_DIR 2>/dev/null
		if [ "$USB_SPINDOWN" = 1 ]; then
			hdparm -S 6 $USB_DEVICE >>$LOGFILE 2>&1
		fi
		sleep 20
		umount $USB_DIR 2>/dev/null
	fi
}

function ftp_backup () {
	if [ "$BACKUP_TO_FTP" = 1 ]; then
		printlog info "================= Start backup to FTP server $FTP_HOST at `date` ============="
		ping -c 2 $FTP_HOST >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			sleep 30
			ping -c 2 $FTP_HOST >/dev/null 2>&1
			if [ $? -ne 0 ]; then
				sleep 30
				ping -c 2 $FTP_HOST >/dev/null 2>&1
				if [ $? -ne 0 ]; then
					if [ -n "$SMB_MAIL" ]; then
						# send email to advise about SMB host
						echo "$SMB_BODY" | mail -s "$SMB_SUBJECT" $SMB_MAIL
						errorsinc #Increment variable errors
					fi
					printlog warn "ERROR... FTP server $FTP_HOST does not respond!" >>$LOGFILE
					printlog warn "ERROR... Skip ftp_backup()" >>$LOGFILE
					printlog warn "Error connecting NAS"
					return
				fi
			fi
		fi
		# check limit rate
		if [ -z "${FTP_RATE}" ]; then
			FTP_RATE=60000000	#Limit rate in byte/sec (default 60Mbyte/sec)
		fi
		unset FTP_PROXY ALL_PROXY HTTPS_PROXY HTTP_PROXY NO_PROXY all_proxy https_proxy no_proxy ftp_proxy http_proxy	

		# Start copying backup
		if [ "$FTP_FUSE" = 1 ]; then
			modprobe fuse
			if [ ! -d /mnt/ftp ]; then
				mkdir -p /mnt/ftp  2>>${LOGFILE}
			fi
			curlftpfs "ftp://$FTP_USER:$FTP_PASS@$FTP_HOST" /mnt/ftp 2>>${INFOFILE}
			ret=$?
			if [ $ret -ne 0 ]; then
				umount /mnt/ftp 2>/dev/null
				sleep 5
				curlftpfs "ftp://$FTP_USER:$FTP_PASS@$FTP_HOST" /mnt/ftp 2>&1 >>${INFOFILE}
				ret=$?
			fi
			if [ $ret -eq 0 ]; then
				#remove old files
				cd "/mnt/ftp/$FTP_DIR" 
				if [ $? -ne 0 ]; then
					printlog warn "Destination FTP directory $FTP_DIR does not exist: create it"
					mkdir -p "/mnt/ftp/$FTP_DIR"
				fi
				cd "/mnt/ftp/$FTP_DIR" 2>/dev/null
				if [ $? -eq 0 ]; then
				 	#directory FTP exists	
					ftp_backup_pre 2>/dev/null
					#nice -$NICE find "/mnt/ftp/$FTP_DIR" -type f -mmin +$(( $FTP_MAXBACKUPTIME * 24 * 60 - 720 )) -exec rm {} \; 2>&1 >>${LOGFILE}
					purgedir /mnt/ftp/$FTP_DIR $FTP_MAXBACKUPTIME
					for dir in weekly daily logs mysql ${BACKUP_OTHER_DIRS}; do
						if [ -f "/mnt/ftp/$FTP_DIR/$dir" ]; then
							rm "/mnt/ftp/$FTP_DIR/$dir" 
						fi
						if [ ! -d "/mnt/ftp/$FTP_DIR/$dir" ]; then
							mkdir "/mnt/ftp/$FTP_DIR/$dir"  2>>${LOGFILE}
						fi
					done
#					for dir in weekly daily logs mysql; do
#						cd $BACKUP_DIR/$dir
#						for file in `find -maxdepth 1 -type f -mtime -$FTP_MAXBACKUPTIME`; do
#							file="`echo $file |sed 's/^\.\///'`"
#							cp -uv $file /mnt/ftp/$dir 2>>$LOGFILE  >>${INFOFILE}
#						done
#					done
#					sleep 16
					sleep 1
					cd /tmp
					umount /mnt/ftp
				else
					printlog warn "Impossible to create directory $FTP_DIR on FTP server"
					umount /mnt/ftp 2>/dev/null
					sleep 5
				fi
			else
				printlog warn "Impossible to mount ftp filesystem using curlftpfs"
				cd /tmp
				umount /mnt/ftp 2>/dev/null
				sleep 5
			fi
		fi
		#FTP_FUSE=0
		if [ -n "${FTP_LOCAL_DIR}" ]; then
			if [ -d "${FTP_LOCAL_DIR}" ]; then
				cd "${FTP_LOCAL_DIR}"
			fi
		fi
		if [ -z "${FTP_LOCAL_DIR}" ]; then
			FTP_LOCAL_DIR=${BACKUP_DIR}
		fi

		if [ -x /usr/bin/lftp ]; then
			ftptype=lftp
			if [ "${FTP_NORESUME}" = "1" ]; then
				ftp_resume=''
			else
				ftp_resume='-c'
			fi
		elif [ -x /usr/bin/wput ]; then
			ftptype=wput
		fi
		MAXSIZE=2147483640	#2GB
		MAXSIZE=214748364000	#200GB
		IFS="$nl"
		for dir in weekly daily logs mysql ${BACKUP_OTHER_DIRS}; do
			cd ${FTP_LOCAL_DIR}/$dir
			if [ "${ftptype}" == "wput" ]; then
				for file in `ls -t`; do
					if [ ! -f $file ]; then
						continue
					fi
					filecp="`echo $file |sed 's/^\.\///'`"
					filesize="`ls -l $filecp |awk '{print $5}'`"
					if [ $filesize -lt $MAXSIZE ]; then
						wput -nv -N -t 2 -l ${FTP_RATE} -B $filecp "ftp://$FTP_USER:$FTP_PASS@$FTP_HOST/$FTP_DIR/$dir/" >>${INFOFILE} 2>>$LOGFILE
					else
						#file bigger than 2GB => split?
						split -b $MAXSIZE ${filecp} ${filecp}_
						for file in ${filecp}_??; do
							wput -nv -N -t 2 -l ${FTP_RATE} -B $file "ftp://$FTP_USER:$FTP_PASS@$FTP_HOST/$FTP_DIR/$dir/" >>${INFOFILE} 2>>$LOGFILE
							rm $file
						done
					fi
				done
			elif [ "${ftptype}" == "lftp" ]; then
				#create script file
				echo "open -u ${FTP_USER},${FTP_PASS} ${FTP_HOST}" >/tmp/backup.lftp
				chmod 600 /tmp/backup.lftp
				echo -e "set net:limit-rate ${FTP_RATE}\nset ftp:ssl-allow no" >>/tmp/backup.lftp
				echo -e "cd ${FTP_DIR}/${dir}\nlcd ${FTP_LOCAL_DIR}/${dir}" >>/tmp/backup.lftp
				#echo -e "mirror -R -c --verbose=0\nquit" >>/tmp/backup.lftp
				for file in `find -maxdepth 2 -type f -mtime -$FTP_MAXBACKUPTIME`; do
					file="`echo $file |sed 's/^\.\///'`"
					echo "put ${ftp_resume} $file" >>/tmp/backup.lftp
				done
				echo "quit" >>/tmp/backup.lftp
				lftp -f /tmp/backup.lftp >>${INFOFILE} 2>>${LOGFILE}
			fi
		done
		if [ "$FTP_FUSE" = 1 ]; then
			curlftpfs "ftp://$FTP_USER:$FTP_PASS@$FTP_HOST/$FTP_DIR" /mnt/ftp 2>&1 >>${INFOFILE}
			ftp_backup_post 2>/dev/null
			printlog info "============= FTP backup space: ================"
			du --max-depth=1 /mnt/ftp >>${INFOFILE}
			sleep 1
			umount /mnt/ftp
		fi
	fi	
}

function ftp_backup2 () {
	# Make backup to another FTP server
	BACKUP_TO_FTP=${BACKUP_TO_FTP2}
	FTP_MAXBACKUPTIME=${FTP_MAXBACKUPTIME2}
	FTP_FUSE=${FTP_FUSE2}
	FTP_HOST=${FTP_HOST2}
	FTP_USER=${FTP_USER2}
	FTP_PASS=${FTP_PASS2}
	FTP_DIR=${FTP_DIR2}
	FTP_LOCAL_DIR=${FTP_LOCAL_DIR2}
	FTP_RATE=${FTP_RATE2}
	FTP_NORESUME=${FTP_NORESUME2}
	ftp_backup
}	

function mysql_backup () {
	#Backup MYSQL DATABASES ?
	if [ "$BACKUP_MYSQL" = "1" ]; then
		if [ -f "${BACKUP_DIR}/mysql" ]; then
			rm "${BACKUP_DIR}/mysql"
		fi
		if [ ! -d "${BACKUP_DIR}/mysql" ]; then
			mkdir -p "$BACKUP_DIR/mysql"  2>>${LOGFILE}
			chmod 700 "$BACKUP_DIR/mysql"
		fi
		printlog info "*** Backup databases:"
		for db in `echo 'show databases;' |mysql -u $MYSQL_USER --password=$MYSQL_PASS |egrep -v '^Database$'`; do 
			if [ "$db" != "information_schema" ]; then
				printlog info -n "    DB $db: "
				file="${BACKUP_DIR}/mysql/backup_mysql_${db}_${filesuffix}.sql.gz"
				mysqldump -u $MYSQL_USER --password=$MYSQL_PASS -q --compact -B --add-drop-database --add-drop-table -R $db 2>/tmp/mysqldump.err |gzip -c > ${file}
				if [ -s /tmp/mysqldump.err ]; then
					egrep -v '(command denied|event|Using a password|LOCK TABLES)' /tmp/mysqldump.err >/tmp/mysqldump.err2
					cat /tmp/mysqldump.err2 >/tmp/mysqldump.err
					rm /tmp/mysqldump.err2
				fi
				if [ -s /tmp/mysqldump.err ]; then
					printlog warn "ERROR: `cat /tmp/mysqldump.err`"
					cat /tmp/mysqldump.err >>$LOGFILE
					errorsinc
				else
					size=`ls -l ${file} |cut -d ' ' -f 5`
					totalsize=$(( ${totalsize} + ${size} ))
					size=`ls -lh ${file} |cut -d ' ' -f 5`
					printlog info ${size}
				fi
			fi
		done
		echo -e "\n" >>$INFOFILE
		find ${BACKUP_DIR}/mysql -type f -mtime +${MYSQL_MAXDAYS} -exec rm {} \;  2>>${LOGFILE} >/dev/null
	fi
}

function pgsql_backup () {
	#Backup PGSQL DATABASES ?
	if [ -n "`ps ax |grep postgres|grep -v grep`" ]; then
		if [ -f "${BACKUP_DIR}/mysql" ]; then
			rm "${BACKUP_DIR}/mysql"
		fi
		if [ ! -d "${BACKUP_DIR}/mysql" ]; then
			mkdir -p "$BACKUP_DIR/mysql"  2>>${LOGFILE}
			chmod 700 "$BACKUP_DIR/mysql"
		fi
		echo -n "*** Backup pgsql db " >>$INFOFILE
		if [ -x /usr/bin/pg_dumpall ]; then
			su postgres -c 'pg_dumpall -c -U postgres' |gzip > ${BACKUP_DIR}/mysql/backup_pgsql_${filesuffix}.sql.gz 
		else
			echo "Postgres database found, but pg_dumpall not installed!" >>$LOGFILE
			errorsinc
		fi
		find ${BACKUP_DIR}/mysql -type f -mtime +${MYSQL_MAXDAYS} -exec rm {} \;  2>>${LOGFILE} >/dev/null
	fi
}


########################################################
##################### Main  ############################

errors=0
totalsize=0
mv ${LOGFILE} ${LOGFILE}.old
mv ${INFOFILE} ${INFOFILE}.old
printlog warn "BACKUP STARTED AT `date`"
printlog info "cmd=$*"
chmod 600 ${LOGFILE} ${INFOFILE}

#Check that no other backup.sh processes are active
if [ -f "/tmp/backup.pid" ]; then
	# backup.sh already active
	pid="`cat /tmp/backup.pid`"
	if [ -n "$pid" -a -z "`ps ax |grep $pid |grep -v grep`" ]; then
		# backup.pid was not cancelled: backup.sh is not executing
		rm /tmp/backup.pid
	else
		errorsinc	#Increment variable errors
		if [ -x /usr/bin/mutt -a -s /var/log/backup.log ]; then
			ps aux |mutt -a /var/log/backup.log.old -a /var/log/backup.info.old -s "Backup error @`hostname`: backup.sh already active PID=$pid" -- $EMAIL_ADMIN
		else
			ps aux |mail -s "Backup error @`hostname`: backup.sh already active PID=$pid. MUTT or backup.log not found" $EMAIL_ADMIN
		fi
		if [ -n "`ps ax |grep -v grep |grep $pid |grep '/usr/local/sbin/backup.sh'`" ]; then
			printlog warn "backup.sh already run, with pid $pid: try to kill old backup.sh"
			kill $pid
			umount /mnt/ftp 2>/dev/null
			umount /mnt/smb 2>/dev/null
			umount /mnt/fromsmb 2>/dev/null
			killall lftp 2>/dev/null
			killall mount.cifs 2>/dev/null
			sleep 1
			umount /mnt/ftp 2>/dev/null
			umount /mnt/smb 2>/dev/null
			umount /mnt/fromsmb 2>/dev/null
			killall -9 lftp 2>/dev/null
			killall -9 mount.cifs 2>/dev/null
			kill -9 $pid
			kill -9 $pid
			umount /mnt/ftp 2>/dev/null
			umount /mnt/smb 2>/dev/null
			umount /mnt/fromsmb 2>/dev/null
			sleep 10
		fi
	fi
fi
echo "$$" >/tmp/backup.pid

cd /
EXCLUDEDIR=''
for dir in `echo $EXCLUDEDIRCHECK`; do
	if [ -d $dir ]; then
		EXCLUDEDIR="${EXCLUDEDIR}${dir} "
	fi
done
EXCLUDE=''
for dir in ${EXCLUDEDIR}; do
	EXCLUDE="$EXCLUDE --exclude $dir"
done

if [ -f /usr/local/sbin/backup_init.sh ]; then
  printlog info "Start backup_init.sh..."
	. /usr/local/sbin/backup_init.sh 2>>${LOGFILE} >>${INFOFILE}
	printlog info "backup_init.sh end"
fi
				
if [ -n "$BACKUP_DIR_MOUNT" ]; then
	if [ -n "`mount |grep $BACKUP_DIR`" ]; then
		printlog warn  "$BACKUP_DIR already mounted: umount $BACKUP_DIR_MOUNT"
		$BACKUP_DIR_UMOUNT 2>&1 >>$INFOFILE
		sleep 5
	fi
	printlog info "Mount backup device: $BACKUP_DIR_MOUNT"
	$BACKUP_DIR_MOUNT	>>"$LOGFILE" 2>&1 >>$INFOFILE
	if [ -z "`mount |grep ${BACKUP_DIR_CONTROL}`" ]; then
		if [ -n "${BACKUP_DIR_MOUNT2}" ]; then
			#2nd try
			BACKUP_PARTITION=${BACKUP_PARTIION2}
			$BACKUP_DIR_MOUNT2 >>"$LOGFILE" 2>&1 >>$INFOFILE
		fi
		if [ -z "`mount |grep ${BACKUP_DIR_CONTROL}`" ]; then
			errorsinc #Increment variable errors
			printerror "Error mounting backup directory:\n cmd=$BACKUP_DIR_MOUNT\n cmd2=${BACKUP_DIR_MOUNT2}\n`mount`"
		fi
	fi
fi

if [ ! -d ${BACKUP_DIR} ]; then 
	mkdir -p ${BACKUP_DIR}  2>>${LOGFILE}
	chgrp backup ${BACKUP_DIR}
	chmod 2750 ${BACKUP_DIR}
fi
cd "$BACKUP_DIR" || printerror "directory $BACKUP_DIR non esiste!" ""
cd /
cmd="$1"
if [ "$cmd" = "-weekly" ]; then
	WEEKLY=1
else
	WEEKLY=0
fi

set $(date)
filesuffix="$6-$2-$3"
currentday="`date +%w`"

if [ -f /usr/local/etc/backup.dir ]; then
	dirfile="/usr/local/etc/backup.dir"
elif [ -f /usr/local/sbin/backup.dir ]; then
	dirfile="/usr/local/sbin/backup.dir"
else
	errorsinc #Increment variable errors
	printerror "Directory file /usr/local/etc/backup.dir does not exist"	#exit
fi		


if [ "$cmd" = "-fromsmb" ]; then
	backup_from_smb
elif [ "$cmd" = "-tosmb" ]; then
	BACKUP_TO_SMB=1
	smb_backup
	smb_backup2
elif [ "$cmd" = "-topartition" ]; then
	BACKUP_TO_PARTITION=1
	partition_backup
elif [ "$cmd" = "-tousb" ]; then
	BACKUP_TO_USB=1
	usb_backup
elif [ "$cmd" = "-toftp" ]; then
	ftp_backup
	ftp_backup2
else
	if [ ${#WEEKLY_BACKUP_DAY} -ne 1 ]; then 
		# WEEKLY_BACKUP_DAY not found or incorrectly set
		WEEKLY_BACKUP_DAY=1
	fi
	for dir in weekly daily logs mysql; do
		if [ -f "${BACKUP_DIR}/$dir" ]; then
			rm "${BACKUP_DIR}/$dir"
		fi
		if [ ! -d "${BACKUP_DIR}/$dir" ]; then
			mkdir -p "$BACKUP_DIR/$dir"  2>>${LOGFILE}
		fi
	done
	if [ "$currentday" = "$WEEKLY_BACKUP_DAY" -o "$WEEKLY" = "1" ]; then
		# weekly a full backup of all data and config. settings:
		#
		IFS="$nl"
		if [ -f "${BACKUP_DIR}/weekly/dvd_pad" ]; then
			rm "${BACKUP_DIR}/weekly/dvd_pad"
		fi
		
		find ${BACKUP_DIR}/weekly -type f -cmin +$MAXBACKUPTIME -exec rm {} \;	2>&1 >/dev/null
		if [ -n "$WEEKLYBACKUPS" ]; then
			# store only $WEEKLYBACKUPS backups
			days=$(( ($WEEKLYBACKUPS-1) * 7 + 1))
			find ${BACKUP_DIR}/weekly -type f -mtime +$days -exec rm {} \;  2>&1 >/dev/null
		fi
		
		if [ -n "${DISKS}" ]; then
			printlog info "Save partition tables into backup_partitions.txt ..."
			file="${BACKUP_DIR}/weekly/backup_partitions.txt"
			> ${file}
			for d in ${DISKS}; do
				echo "========== Disk $d ==========" >>${file}
				sfdisk -d /dev/$d >>${file} 2>/dev/null
			done
		fi
		#backup directories like /tmp /dev ....
		cd /
		EXCLUDEDIR="${EXCLUDEDIR} tmp mnt media"
		>$LIST
		find / -maxdepth 1 -type l -print >> $LIST
		for dir in `echo ${EXCLUDEDIR}|tr ' ' "\n"`; do
			if [ -d $dir ]; then
				find $dir -type d -print >> $LIST
				find $dir -type l -print >> $LIST
				find $dir -type s -print >> $LIST
			fi
		done
#		if [ -s $LIST ]; then 
#			nice -$NICE tar cpT $LIST --no-recursion 2>>${LOGFILE} |${COMPRESS} ${COMPRESS_ARG} >"${BACKUP_DIR}/weekly/backup_dirs_$filesuffix.${COMPRESS_SUFFIX}"
#		fi
		nice -$NICE tar cp dev |${COMPRESS} ${COMPRESS_ARG} 2>>${LOGFILE} > "${BACKUP_DIR}/weekly/backup_dev_$filesuffix.${COMPRESS_SUFFIX}"
		
		for dir in `cat ${dirfile}|grep -v '^#'`; do 
			IFS="$space$tab$nl"
			dirfile="backup_`echo "$dir" |tr '/' '_'|tr ' ' '_'`_$filesuffix.${COMPRESS_SUFFIX}"
			if [ -d "$dir" ]; then
				printlog info -n "*** Backup directory $dir: "
				file="${BACKUP_DIR}/weekly/${dirfile}"
				rm -f "${file}"
				nice -$NICE tar cp $EXCLUDE --exclude "$dir/Cestino" --exclude "$dir/logs"  --exclude "$dir/nobackup" "$dir" 2>>"$LOGFILE" | ${COMPRESS} ${COMPRESS_ARG} >"${file}"
				size=`ls -l ${file} |cut -d ' ' -f 5`
				totalsize=$(( ${totalsize} + ${size} ))
				size=`ls -lh ${file} |cut -d ' ' -f 5`
				printlog info ${size}
			fi
		done
		IFS="$space$tab$nl"
####	rm -f ${BACKUP_DIR}/incrbackup*.${COMPRESS_SUFFIX}
		#Backup MYSQL DATABASES ?
		mysql_backup
		pgsql_backup

		if [ -s /usr/local/etc/backup.smb ]; then
			backup_from_smb weekly
		fi
		
		backup_logs	;# perform LOG
		
		#Must copy backup to CDRW?
		if [ "$BACKUP_TO_CDRW" = "1" -o "$BACKUP_TO_DVDRW" = "1" ]; then
			cdrw_backup
		fi
		#backup to TAPE ??
		if [ "$BACKUP_TO_TAPE" == "1" ]; then
		tape_backup
		fi
	else
		# incremental backup:
		if [ ! -d ${BACKUP_DIR}/daily ]; then 
			mkdir -p ${BACKUP_DIR}/daily  2>>${LOGFILE}
			chgrp backup ${BACKUP_DIR}/daily
			chmod 2750 ${BACKUP_DIR}/daily
		fi
		#
		find ${BACKUP_DIR}/daily -type f -mmin +$MAXBACKUPTIME -exec rm {} \;	2>&1 >/dev/null
		IFS="$nl"
		for dir in `cat ${dirfile}|grep -v '^#'`; do
			IFS="$space$tab$nl"
			dirfile="incrbackup_`echo "$dir" |tr '/' '_'`_$filesuffix.${COMPRESS_SUFFIX}"
			if [ -d "$dir" ]; then
				printlog info -n "*** Backup directory $dir: "
				find ${dir} -depth -type f -mmin -1800 -print |egrep -v "($dir/Cestino|$dir/logs|.log\$|cache/)" > $LIST
				if [ -s "${LIST}" ]; then 
					file="${BACKUP_DIR}/daily/${dirfile}"
					nice -$NICE tar cpT "$LIST" $EXCLUDE  --exclude "$dir/nobackup" 2>>"$LOGFILE" | ${COMPRESS} ${COMPRESS_ARG} >"${file}"
					size=`ls -l ${file} |cut -d ' ' -f 5`
					totalsize=$(( ${totalsize} + ${size} ))
					size=`ls -lh ${file} |cut -d ' ' -f 5`
					printlog info ${size}
				else
					printlog info "UNCHANGED"
				fi
				rm -f "$LIST"
			fi
		done
		IFS="$space$tab$nl"
		#Backup MYSQL DATABASES ?
		mysql_backup
		pgsql_backup
		if [ -s /usr/local/etc/backup.smb ]; then
			backup_from_smb incremental
		fi
	fi
	# USB/SMB backup??
	partition_backup
	usb_backup
	smb_backup
	smb_backup2
	ftp_backup
	ftp_backup2
fi

chmod -R o-rwx ${BACKUP_DIR}
#check disk usage
if [ -n "${BACKUP_PARTITION_CHECK}" ]; then
	usage=`df |grep ${BACKUP_PARTITION_CHECK} |awk '{print $5}'|tr -d %`
	if [ -n "$usage" ]; then
		if [ $usage -gt 90 ]; then
			errorsinc #Increment variable errors
			printlog warn "Partition ${BACKUP_PARTITION_CHECK} is full"
			df |mail -s "backup: partition ${BACKUP_PARTITION_CHECK} full" $EMAIL_ADMIN
		fi
	fi
fi
cd /

if [ -x /usr/local/sbin/backup_end.sh ]; then
	/usr/local/sbin/backup_end.sh
fi

$BACKUP_DIR_UMOUNT 2>/dev/null
$BACKUP_DIR_AFTER_UMOUNT >>${INFOFILE} 2>&1
#rm -f "$LOGFILE"
printlog warn "BACKUP TERMINATED AT `date`"
rm -f /tmp/backup.pid

if [ $errors -gt 0 ]; then
	if [ "a${cmd:0:3}" != "a-to" ]; then
		# -tosmb or -tousb or -toftp not specified => send mail with errors
		printerror "backup exits with $errors errors"
	fi
fi

egrep -vi '(mysql.event|socket ignor|tar: Removing leading|tar: Rimuovo|file changed as|mentre lo stavo leggendo|failed to preserve ownership|impossibile preservare|Hard link count is wrong|Certificate verification)' ${LOGFILE} >/tmp/backup.log.tmp
cat /tmp/backup.log.tmp >${LOGFILE}
rm /tmp/backup.log.tmp

egrep -vi '(mysql.event|socket ignor|tar: Removing leading|tar: Rimuovo|file changed as|mentre lo stavo leggendo|failed to preserve ownership|impossibile preservare|Hard link count is wrong|Certificate verification)' ${INFOFILE} >/tmp/backup.log.tmp
cat /tmp/backup.log.tmp >${INFOFILE}
rm /tmp/backup.log.tmp

printlog info "Today backup size: ${totalsize} bytes"
if [ -n "${EMAIL_INFO}" ]; then
	echo -e "`cat $LOGFILE`\n===================================\n`cat $INFOFILE`" |mail -s "Backup information" ${EMAIL_INFO}
fi
if [ -n "${BACKUP_LASTCMD}" ]; then
	if [ $# -eq 0 ]; then
		echo "Executing command ${BACKUP_LASTCMD} ..." >>${LOGFILE}
		sleep 10
		${BACKUP_LASTCMD}
	fi
fi
